<?php

require_once 'commpass_utils.php';
require_once 'lib/utils.php';

if(!function_exists('hd_print'))
{
    function hd_print($arg)
    {
        echo "$arg\n";
    }
}

class AresApiException extends Exception {
    public function __construct($message, $code = 0, $previous = null)
    {
	parent::__construct($message, $code, $previous);
    }
}

class MwAuthException extends AresApiException {
    public function __construct($message, $code = 0, $previous = null)
    {
        parent::__construct($message, $code, $previous);
    }
}

class MwBlockedException extends AresApiException {
    public function __construct($message, $code = 0, $previous = null)
    {
        parent::__construct($message, $code, $previous);
    }
}


class AresApi {
    private $api_url;
    private $mac;
    private $plugin_cookies;
    
    public function __construct($api_url, $mac, &$plugin_cookies)
    {
        $this->api_url = $api_url;
        $this->mac = $mac;
        $this->plugin_cookies = &$plugin_cookies;
    }
    
    public function login($params = array())
    {
        $params['require_session'] = true;
        $result = $this->_RpcRequest('login', $params);
        $this->plugin_cookies->session_id = $result['session_id'];
        $this->plugin_cookies->session_time = time();
        
    }
    
    private function _RpcRequest($method, $params = array())
    {
        $params['macaddr'] = $this->mac;
        
        if($method != 'login') 
            if(isset($this->plugin_cookies->session_id) && $this->plugin_cookies->session_id)
            {
                $params['session_id'] = $this->plugin_cookies->session_id;
            }
            else
            {
                hd_print("Warning: session_id is not set");
            }
        
        $data = HD::make_json_rpc_request($method, $params);
        $result_raw = http_post_document($this->api_url, json_encode($data));
        $result = json_decode($result_raw, true);
        if(!$result) throw new AresApiException('Bad response from MW server: json_decode() failed ');
        if(isset($result['result'])) return $result['result'];
        if(isset($result['error']['message']) && isset($result['error']['code']))
        {
            hd_print("Got API exception: " . $result['error']['message'] . ", code: " . $result['error']['code']);
            switch(strval($result['error']['code']))
            {
                case '-32002':
                    hd_print('Authentication error');
                    throw new MwAuthException($result['error']['message'], $result['error']['code']);
                case '-32003':
                    hd_print('STB disabled');
                    throw new MwBlockedException($result['error']['message'], $result['error']['code']);
                // FIXME: add more exceptions
                default:
                    hd_print('Error code is unknown');
            }
        }
        
        throw new AresApiException(isset($result['error']['message']) ? $result['error']['message'] : 'unknown error', 
                isset($result['error']['code']) ? $result['error']['code'] : 0);
    }
    
    public function __call($method, $params = false)
    {
        $args = (isset($params[0])) ? $params[0] : array();
        return $this->_RpcRequest($method, $args);
    }
    
}

