<?php
//die('BROKEN');
///////////////////////////////////////////////////////////////////////////

require_once 'lib/action_factory.php';
require_once 'lib/dune_exception.php';

///////////////////////////////////////////////////////////////////////////

class CommpassPluginFw extends DunePluginFw
{
    public static $plugin_class_name = null;

    ///////////////////////////////////////////////////////////////////////

    protected function print_backtrace(&$ex)
    {
        $trace = array_reverse(explode("\n", $ex->getTraceAsString()));
        array_shift($trace);
        array_pop($trace);
        $result = array();
        for ($i = 0; $i < count($trace); $i++)
        {
            $result[] = ($i + 1) . ')' . substr($trace[$i], strpos($trace[$i], ' '));
        }

        hd_print("\t" . implode("\n\t", $result));
    }

    public function create_plugin()
    {
        return new CommpassPluginFw::$plugin_class_name;
    }

    ///////////////////////////////////////////////////////////////////////

    public function call_plugin($call_ctx_json)
    {
        // $call_ctx = json_decode($call_ctx_json);
        // $ret = $this->call_plugin_impl($call_ctx);
        // return json_encode($ret);

        return
            json_encode(
                $this->call_plugin_impl(
                    json_decode($call_ctx_json)));
    }

    ///////////////////////////////////////////////////////////////////////

    protected function call_plugin_impl($call_ctx)
    {
        static $plugin;

        if (is_null($plugin))
        {
            try
            {
                hd_print('Instantiating plugin...');
                $plugin = $this->create_plugin();
                hd_print('Plugin instance created.');
            }
            catch (Exception $e)
            {
                hd_print('Error: can not instantiate plugin (' . $e->getMessage() . ')');
                $this->print_backtrace($e);
                return
                    array
                    (
                        PluginOutputData::has_data => false,
                        PluginOutputData::plugin_cookies => $call_ctx->plugin_cookies,
                        PluginOutputData::is_error => true,
                        PluginOutputData::error_action =>
                            ActionFactory::show_error(
                                true,
                                'System error',
                                array(
                                    'Не удалось запустить плагин',
                                    'Пожалуйста, обратитесь в службу',
                                    'технической поддержки.'))
                    );
            }
        }

        // assert($plugin);

        $out_data = null;

        try
        {
            $out_data = $this->invoke_operation($plugin, $call_ctx);
        }
        catch(MwAuthException $e)
        {
            hd_print("Got MwAuthException exception");
            throw new ShowRegistrationScreenException();
        }
        catch (DuneException $e)
        {
            hd_print("Error: DuneException caught: " . $e->getMessage());
            return
                array
                (
                    PluginOutputData::has_data => false,
                    PluginOutputData::plugin_cookies => $call_ctx->plugin_cookies,
                    PluginOutputData::is_error => true,
                    PluginOutputData::error_action => $e->get_error_action()
                );
        }
	catch (UpdateFoldersMessageException $e)
	{
            hd_print("UpdateFoldersMessageException caught: " . $e->getMessage());
	}
        catch (Exception $e)
        {
            hd_print("Error: Fatal: Exception caught: " . $e->getMessage());

            $this->print_backtrace($e);

            return
                array
                (
                    PluginOutputData::has_data => false,
                    PluginOutputData::plugin_cookies => $call_ctx->plugin_cookies,
                    PluginOutputData::is_error => true,
                    PluginOutputData::error_action =>
                        ActionFactory::show_error(
                            true,
                            'System error',
                            array(
                                'Внутреняя ошибка плагина',
                                'Пожалуйста, перезапустите плагин.',
                                'Если ошибка повториться, пожалуйста,',
                                'обратитесь в службу технической поддержки')
                                )
                );
        }

        // Note: change_tv_favorites() may return NULL even if it's completed
        // successfully.

        $plugin_output_data = array
        (
            PluginOutputData::has_data => !is_null($out_data),
            PluginOutputData::plugin_cookies => $call_ctx->plugin_cookies,
            PluginOutputData::is_error => false,
            PluginOutputData::error_action => null
        );

        if ($plugin_output_data[PluginOutputData::has_data])
        {
            $plugin_output_data[PluginOutputData::data_type] =
                $this->get_out_type_code($call_ctx->op_type_code);

            $plugin_output_data[PluginOutputData::data] = $out_data;
        }

        return $plugin_output_data;
    }
}

///////////////////////////////////////////////////////////////////////////

DunePluginFw::$instance = new CommpassPluginFw();

///////////////////////////////////////////////////////////////////////////
