<?php

///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';

require_once 'commpass_channel.php';
//require_once 'update_firmware.php';
require_once 'messages.php';
require_once 'ares_api_wrapper.php';

///////////////////////////////////////////////////////////////////////////

class CommpassTv extends AbstractTv implements UserInputHandler 
{

    private $mac;
    private $profiles;
    
    public function __construct()
    {
        parent::__construct(
                AbstractTv::MODE_CHANNELS_N_TO_M, CommpassConfig::TV_FAVORITES_SUPPORTED, false);
        $this->mac = get_mac_address();
        $this->profiles = array();
        
        // playback events capture
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    /*** UserInputHandler implementation ***/
    public function get_handler_id()
    {
        return 'CommpassTvClassInstance';
    }

    // Not really user input handler, but TV Playback events handler
    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        switch($user_input->control_id)
        {
            case 'timer':
                return $this->handle_timer_event($user_input, $plugin_cookies);
            case 'playback_stop':
                return $this->handle_playback_stop_event($user_input, $plugin_cookies);
            default:
                hd_print('Got unknown event: \'' . $user_input->control_id . '\'');
        }
    }
    /*** End of UserInputHandler implementation  ***/

    private function mwapi_call($method, &$plugin_cookies, $args = array())
    {
        return AresApiWrapper::Call($method, $plugin_cookies, $args);
    }

    private function handle_timer_event($user_input, &$plugin_cookies)
    {
        hd_print('handle_timer_event');
        
        $this->unload_channels();
        $this->load_channels($plugin_cookies);
        
        $actions = array();
	$actions[GUI_EVENT_TIMER] = UserInputHandlerRegistry::create_action($this, 'timer');
	return array(
            GuiAction::handler_string_id => CHANGE_BEHAVIOUR_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data => array(
                ChangeBehaviourActionData::post_action => ActionFactory::invalidate_folders(array(), null), 
                ChangeBehaviourActionData::actions => $actions,
                ChangeBehaviourActionData::timer => array(GuiTimerDef::delay_ms => CommpassConfig::UPDATE_CHANNELS_INTERVAL * 1000)
            ),
            GuiAction::params => array(
                'handler_id' => $this->get_handler_id(),
                'control_id' => 'timer'
            )
        );
    }
    
    private function handle_playback_stop_event($user_input, &$plugin_cookies)
    {
        hd_print('handle_playback_stop_event');
    }
    
    public function get_fav_icon_url()
    {
        return CommpassConfig::FAV_CHANNEL_GROUP_ICON_PATH;
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
    {
        $info = parent::get_tv_info($media_url, $plugin_cookies);
        $info[PluginTvInfo::timer] = array(GuiTimerDef::delay_ms => CommpassConfig::UPDATE_CHANNELS_INTERVAL * 1000);
        $info[PluginTvInfo::actions] = array(
            GUI_EVENT_TIMER => UserInputHandlerRegistry::create_action($this, 'timer'),
            GUI_EVENT_PLAYBACK_STOP => UserInputHandlerRegistry::create_action($this, 'playback_stop')
        );
        
        return $info;
    }
    
    protected function get_protect_code($age_limit)
    {
        if(isset($this->profiles[$age_limit]) && $this->profiles[$age_limit]['require_password'])
        {
            return (isset($this->profiles[$age_limit]['password']) && $this->profiles[$age_limit]['password']) 
                    ? $this->profiles[$age_limit]['password'] : CommpassConfig::DEFALT_PROFILE_PASSWORD;
        }
        return false;
    }
    
    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {
        $this->ensure_channels_loaded($plugin_cookies);
        $channel = $this->get_channel($channel_id);
        $url = new Url(($archive_ts > 0) ? $channel->get_archive_url() : $channel->get_streaming_url());
        
        if($archive_ts > 0) // archive
        {
            // We support flussonic HLS only
            //$length = time() - $archive_ts;
            $url->set_path($url->get_path() . "/mono-$archive_ts-now.m3u8");
            if(isset($plugin_cookies->session_id)) $url->set_query('token=' . $plugin_cookies->session_id);
        }
        
        if($channel->is_protected())
        {
            $url->set_query($url->get_query() . "&protect_code=$protect_code" . "&age_limit=" . $channel->get_age_limit());
        }

        hd_print("Select channel id=$channel_id, title=\"" . $channel->get_title() . "\", age_limit=" .
                $channel->get_age_limit() . ", playback_url = " . $url->to_string());
        return $url->to_string();
    }

    public function get_tv_stream_url($media_url, &$plugin_cookies)
    {
        $url = new Url($media_url);
        $query = array();
        foreach(explode('&', $url->get_query()) as $pair)
        {
            $a = explode('=', $pair, 2);
            if(!count($a)) continue;
            $query[$a[0]] = (count($a) > 1)?$a[1] : '';
        }

        if(isset($query['protect_code']) && isset($query['age_limit']))
        {
            if($query['protect_code'] != $this->get_protect_code($query['age_limit']))
            {
                hd_print("Wrong protect code: " . $query['protect_code']);
                return 'protected';
            }
            hd_print("Protect code is ok (" . $query['protect_code'] . ")");
            unset($query['protect_code']);
            unset($query['age_limit']);
            $url->set_query(implode('&', $query));
        }
        
        // flussonic multi-language URL
        $stream_url = preg_replace('/index\.m3u8/', 'mono.m3u8', $url->to_string());
        hd_print("stream_url: $stream_url");
        return $stream_url;
    }


    protected function load_channels(&$plugin_cookies)
    {

hd_print("load_channels()");        
        //check_update_firmware(true);

        $wifi = get_mac_address('wlan0');
        if ($wifi)
            warn_wifi($plugin_cookies);

            $this->profiles = array();
            foreach($this->mwapi_call('get_profiles', $plugin_cookies) as $profile)
                $this->profiles[$profile['age_limit']+0] = $profile;
            
            $result = $this->mwapi_call('get_playlists', $plugin_cookies);
            $this->channels = new HashedArray();
            $this->groups = new HashedArray();

            if ($this->is_favorites_supported())
            {
                $this->groups->put(
                        new FavoritesGroup(
                        $this, '__favorites', CommpassConfig::FAV_CHANNEL_GROUP_CAPTION, CommpassConfig::FAV_CHANNEL_GROUP_ICON_PATH));
            }

            $this->groups->put(
                    new AllChannelsGroup(
                    $this, CommpassConfig::ALL_CHANNEL_GROUP_CAPTION, CommpassConfig::ALL_CHANNEL_GROUP_ICON_PATH));
            
            // channels
            if (isset($result['medialist']))
            {
                $now = new DateTime('now');
                $past_epg_days = 0;
                $future_epg_days = 0;
                $number = 0;

                
                foreach ($result['medialist'] as $item)
                {
                    $epg_start = HD::parse_timestamp($item['epg_start']);
                    $epg_stop = HD::parse_timestamp($item['epg_stop']);
                    if (is_null($epg_start) || is_null($epg_stop))
                    {
                        $past_epg_days = 0;
                        $future_epg_days = 0;
                    } else
                    {
                        $epg_start_diff = $now->diff($epg_start);
                        $epg_stop_diff = $now->diff($epg_stop);
                        $past_epg_days = $epg_start_diff->days;
                        $future_epg_days = $epg_stop_diff->days;
                    }
                    
                    $channel = new CommpassChannel(
                            strval($item['id']), 
                            strval($item['name']), 
                            strval($item['logo']), 
                            strval($item['mrl']), 
                            strval($item['timeshift_url']),
                            intval($number++), // FIXME
                            intval($item['epg_start']), 
                            intval($item['epg_stop']), 
                            intval($item['timeshift_archive_length']),
                            0, 
                            intval($item['age_limit']),
                            $this->get_protect_code(intval($item['age_limit'])) !== false);
                    $this->channels->put($channel);
                }
            }
            
            // channel groups (one group is `one playlist` in the ARES terms)
            if (isset($result['playlists']) && is_array($result['playlists']))
            {
                foreach ($result['playlists'] as $item)
                {
                    if (!isset($item['items']) || !is_array($item['items']) || !count($item['items']))
                        continue;

                    $group = new DefaultGroup($item['id'], $item['name'], $item['logo']);
                    $this->groups->put($group);
                    foreach ($item['items'] as $id)
                    {
                        $channel = $this->channels->get(strval($id));
                        if (!$channel)
                            continue;
                        $channel->add_group($group);
                        $group->add_channel($channel);
                    }
                }
            }

            if(function_exists('CommpassConfig::sort_channels_cb'))
                $this->channels->usort('CommpassConfig::sort_channels_cb');
            
/*
            // fix для клиентов, у которых нет доступа к NTP
            // без установки правильного времени они не получат доступ к VCAS и EPG
            $datetime = $xml->current_time[0];
            if (date('Y') < 2013 && $datetime)
            {
                hd_print("fixing current date and time");
                $output = exec("date -u $datetime");
                hd_print($output);
            }
 */     
 /*       catch (Exception $ex)
        {
            throw new DuneException($ex->getMessage(), $ex->getCode(), 
                    ActionFactory::show_error(true, 'Ошибка соединения с сервером', preg_split('/<br>/', $ex->getMessage())), null);
        }
  * 
  */
    }

    ///////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        $epg = array();
        //$this->MwApi($plugin_cookies);
        $day_start = HD::format_timestamp ($day_start_ts, 'Y-m-d H:i:sO');
        $day_end = HD::format_timestamp ($day_start_ts + 86400, 'Y-m-d H:i:sO');
        $result = $this->mwapi_call('get_epg', $plugin_cookies, array(
            'media_id' => $channel_id,
            'limit' => 999,
            'start' => $day_start,
            'stop'  => $day_end
        ));
        
        if (isset($result['programs']))
        {
            foreach ($result['programs'] as $p)
            {
                $start = HD::parse_timestamp($p['start']);
                $stop = HD::parse_timestamp($p['stop']);
                if (is_null($start) || is_null($stop)) continue;
                $start_ts = $start->getTimestamp();
                $stop_ts = $stop->getTimestamp();
                if ($start_ts < $day_start_ts || $start_ts > $day_start_ts + 86400)
                    continue;
                if (is_null($p['description'])) $p['description'] = '';
                $epg[] = new DefaultEpgItem($p['title'], $p['description'], $start_ts, $stop_ts);
            }
        }
        
        if(!count($epg))
            hd_print("Warning: no day EPG for program id=$channel_id, start_ts=" . $day_start);
        else
            hd_print("Loaded EPG for channel id=$channel_id, start_ts=" . $day_start . ", event count=" . count($epg));
        
        return new EpgIterator ($epg, $day_start_ts, $day_start_ts + 86400);

    }

}

///////////////////////////////////////////////////////////////////////////
