<?php

require_once 'lib/user_input_handler_registry.php';
require_once 'lib/dune_exception.php';
require_once 'commpass_utils.php';


class ShowMessageHandler implements UserInputHandler
{

    private static $instance;

    private function __construct()
    {
    }

    static public function get_instance()
    {
	if(!isset(self::$instance))
	    self::$instance = new self();
	return self::$instance;
    }

    public function get_handler_id()
    {
        return 'show_message_handler';
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
	    return ActionFactory::close_dialog_and_run(ActionFactory::open_folder());
    }

};

UserInputHandlerRegistry::get_instance()->register_handler(ShowMessageHandler::get_instance());


class ShowMessageException extends DuneException
{
    public function __construct(/*string*/$title, /*array of strings*/$message_lines)
    {

	$defs = array();
        if(!is_array($message_lines)) $message_lines = FormatText(100, $message_lines);
	foreach($message_lines as $line)
	{
	    ControlFactory::add_label($defs, null, $line);
	}
        ControlFactory::add_vgap($defs, 50);
	ControlFactory::add_button($defs, ShowMessageHandler::get_instance(), null, 'message_ok_button', null, 'Ok', 300);
        ControlFactory::add_vgap($defs, 50);
	//ControlFactory::add_close_dialog_button($defs, 'Ok', 300);
	$error_action = ActionFactory::show_dialog ($title, $defs, true);

//	$error_action = ActionFactory::show_error(false, $title, $message_lines);
	parent::__construct('show administrative message', $code, $error_action, null);
    }
};

class ShowErrorMessageException extends DuneException
{
    public function __construct(/*string*/$title, /*array of strings*/$message_lines)
    {

	$defs = array();
        if(!is_array($message_lines)) $message_lines = FormatText(100, $message_lines);
/*
        foreach($message_lines as $line)
	{
	    ControlFactory::add_label($defs, null, $line);
	}
*/
	$error_action = ActionFactory::show_error(true, $title, $message_lines);
	parent::__construct('show error message', 0, $error_action, null);
    }
};


class UpdateFoldersMessageException extends DuneException
{
    public function __construct()
    {
	$error_action = ActionFactory::invalidate_folders(array('tv_group_list'));
	parent::__construct('update folders exception', 0, $error_action, null);
    }
};

// TODO
function check_messages($have_gui = false)
{
    $spool = DuneSystem::$properties['tmp_dir_path'] . '/messages';
    if(file_exists("$spool/update_folders"))
    {
	unlink("$spool/update_folders");
	throw new UpdateFoldersMessageException();
    }
}

function warn_wifi(&$cookies)
{
    $cur_time = time();
    if(isset($cookies->warn_wifi_done) && $cookies->warn_wifi_done > $cur_time - 60*60*24*30) 
	return;
    
    $cookies->warn_wifi_done = $cur_time;

    throw new ShowMessageException(
        'Внимание!', 
        array(
    	'Ваша приставка подключена к сети через беспроводное подключение (Wi-Fi).',
	'При данном способе подключения качественная работа телевидения не гарантируется,',
	'возможно периодическое замирание изображения, появление цветных "квадратиков"',
	'и прерывания звука. Для качественной работы телевидения необходимо подключить',
	'приставку к сети по проводу'
        )
    );
}

?>